@@ -25,10 +25,6 @@
 
 #include <glib.h>
 #include <glib/gi18n-lib.h>
-#include <telepathy-glib/account-channel-request.h>
-#include <telepathy-glib/util.h>
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/interfaces.h>
 
 #include "empathy-ft-handler.h"
 #include "empathy-time.h"
