@@ -86,6 +86,21 @@ typedef struct
   gchar *access_token;
 } AuthData;
 
+static AuthData *
+auth_data_new (EmpathyGoaAuthHandler *self,
+    TpChannel *channel,
+    TpAccount *account)
+{
+  AuthData *data;
+
+  data = g_slice_new0 (AuthData);
+  data->self = g_object_ref (self);
+  data->channel = g_object_ref (channel);
+  data->account = g_object_ref (account);
+
+  return data;
+}
+
 static void
 auth_data_free (AuthData *data)
 {
@@ -158,13 +173,13 @@ got_oauth2_access_token_cb (GObject *source,
       case EMPATHY_SASL_MECHANISM_FACEBOOK:
         empathy_sasl_auth_facebook_async (data->channel,
             goa_oauth2_based_get_client_id (oauth2), access_token,
-            auth_cb, NULL);
+            auth_cb, data);
         break;
 
       case EMPATHY_SASL_MECHANISM_WLM:
         empathy_sasl_auth_wlm_async (data->channel,
             access_token,
-            auth_cb, NULL);
+            auth_cb, data);
         break;
 
       default:
@@ -300,10 +315,7 @@ empathy_goa_auth_handler_start (EmpathyGoaAuthHandler *self,
   DEBUG ("Start Goa auth for account: %s",
       tp_proxy_get_object_path (account));
 
-  data = g_slice_new0 (AuthData);
-  data->self = g_object_ref (self);
-  data->channel = g_object_ref (channel);
-  data->account = g_object_ref (account);
+  data = auth_data_new (self, channel, account);
 
   if (self->priv->client == NULL)
     {
