@@ -67,6 +67,11 @@
     || (t1 >= t2 && (t1 - t2) > (G_MAXUINT32/2)) \
   )
 
+enum
+{
+  PROP_INDIVIDUAL_MGR = 1
+};
+
 struct _EmpathyChatWindowPriv
 {
   EmpathyChat *current_chat;
@@ -75,6 +80,7 @@ struct _EmpathyChatWindowPriv
   gboolean dnd_same_window;
   EmpathyChatroomManager *chatroom_manager;
   EmpathyNotifyManager *notify_mgr;
+  EmpathyIndividualManager *individual_mgr;
   GtkWidget *dialog;
   GtkWidget *notebook;
   NotifyNotification *notification;
@@ -112,6 +118,8 @@ struct _EmpathyChatWindowPriv
   GSettings *gsettings_ui;
 
   EmpathySoundManager *sound_mgr;
+
+  gboolean updating_menu;
 };
 
 static GList *chat_windows = NULL;
@@ -593,11 +601,14 @@ chat_window_conversation_menu_update (EmpathyChatWindow *self)
 }
 
 static void
-chat_window_contact_menu_update (EmpathyChatWindow *self,
-    EmpathyChatWindow *window)
+chat_window_contact_menu_update (EmpathyChatWindow *self)
 {
   GtkWidget *menu, *submenu, *orig_submenu;
 
+  if (self->priv->updating_menu)
+    return;
+  self->priv->updating_menu = TRUE;
+
   menu = gtk_ui_manager_get_widget (self->priv->ui_manager,
     "/chats_menubar/menu_contact");
   orig_submenu = gtk_menu_item_get_submenu (GTK_MENU_ITEM (menu));
@@ -624,8 +635,10 @@ chat_window_contact_menu_update (EmpathyChatWindow *self,
     {
       tp_g_signal_connect_object (orig_submenu,
           "notify::visible",
-          (GCallback)_submenu_notify_visible_changed_cb, window, 0);
+          (GCallback)_submenu_notify_visible_changed_cb, self, 0);
     }
+
+  self->priv->updating_menu = FALSE;
 }
 
 static guint
@@ -809,7 +822,7 @@ chat_window_update (EmpathyChatWindow *self,
      menu watching. */
   if (update_contact_menu)
     {
-      chat_window_contact_menu_update (self, self);
+      chat_window_contact_menu_update (self);
     }
 
   chat_window_title_update (self);
@@ -1906,6 +1919,52 @@ chat_window_focus_in_event_cb (GtkWidget *widget,
   return FALSE;
 }
 
+static void
+contacts_loaded_cb (EmpathyIndividualManager *mgr,
+    EmpathyChatWindow *self)
+{
+  chat_window_contact_menu_update (self);
+}
+
+static gboolean
+chat_window_focus_out_event_cb (GtkWidget *widget,
+    GdkEvent *event,
+    EmpathyChatWindow *self)
+{
+  if (self->priv->individual_mgr != NULL)
+    return FALSE;
+
+  /* Keep the individual manager alive so we won't fetch everything from Folks
+   * each time we need to use it. Loading FolksAggregator can takes quite a
+   * while (if user has a huge LDAP abook for example) and it blocks
+   * the mainloop during most of this loading. We workaround this by loading
+   * it when the chat window has been unfocused and so, hopefully, not impact
+   * the reactivity of the chat window too much.
+   *
+   * The individual manager (and so Folks) is needed to know to which
+   * FolksIndividual a TpContact belongs, including:
+   * - empathy_chat_get_contact_menu: to list all the personas of the contact
+   * - empathy_display_individual_info: to invoke gnome-contacts with the
+   *   FolksIndividual.id of the contact
+   * - drag_data_received_individual_id: to find the individual associated
+   *   with the ID we received from the DnD in order to invite him.
+   */
+  self->priv->individual_mgr = empathy_individual_manager_dup_singleton ();
+
+  if (!empathy_individual_manager_get_contacts_loaded (
+      self->priv->individual_mgr))
+    {
+      /* We want to update the contact menu when Folks is loaded so we can
+       * list all the personas of the contact. */
+      tp_g_signal_connect_object (self->priv->individual_mgr, "contacts-loaded",
+          G_CALLBACK (contacts_loaded_cb), self, 0);
+    }
+
+  g_object_notify (G_OBJECT (self), "individual-manager");
+
+  return FALSE;
+}
+
 static gboolean
 chat_window_drag_drop (GtkWidget *widget,
     GdkDragContext *context,
@@ -2001,7 +2060,6 @@ drag_data_received_individual_id (EmpathyChatWindow *self,
     guint time_)
 {
   const gchar *id;
-  EmpathyIndividualManager *manager = NULL;
   FolksIndividual *individual;
   EmpathyTpChat *chat;
   TpContact *tp_contact;
@@ -2026,9 +2084,13 @@ drag_data_received_individual_id (EmpathyChatWindow *self,
       goto out;
     }
 
-  manager = empathy_individual_manager_dup_singleton ();
+  if (self->priv->individual_mgr == NULL)
+    /* Not likely as we have to focus out the chat window in order to start
+     * the DnD but best to be safe. */
+    goto out;
 
-  individual = empathy_individual_manager_lookup_member (manager, id);
+  individual = empathy_individual_manager_lookup_member (
+          self->priv->individual_mgr, id);
   if (individual == NULL)
     {
       DEBUG ("Failed to find individual %s", id);
@@ -2053,7 +2115,6 @@ drag_data_received_individual_id (EmpathyChatWindow *self,
 
 out:
   gtk_drag_finish (context, TRUE, FALSE, time_);
-  tp_clear_object (&manager);
 }
 
 static void
@@ -2216,6 +2277,7 @@ chat_window_finalize (GObject *object)
   g_object_unref (self->priv->gsettings_notif);
   g_object_unref (self->priv->gsettings_ui);
   g_object_unref (self->priv->sound_mgr);
+  g_clear_object (&self->priv->individual_mgr);
 
   if (self->priv->notification != NULL)
     {
@@ -2243,13 +2305,39 @@ chat_window_finalize (GObject *object)
   G_OBJECT_CLASS (empathy_chat_window_parent_class)->finalize (object);
 }
 
+static void
+chat_window_get_property (GObject *object,
+    guint property_id,
+    GValue *value,
+    GParamSpec *pspec)
+{
+  EmpathyChatWindow *self = EMPATHY_CHAT_WINDOW (object);
+
+  switch (property_id)
+    {
+      case PROP_INDIVIDUAL_MGR:
+        g_value_set_object (value, self->priv->individual_mgr);
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
 static void
 empathy_chat_window_class_init (EmpathyChatWindowClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GParamSpec *spec;
 
+  object_class->get_property = chat_window_get_property;
   object_class->finalize = chat_window_finalize;
 
+  spec = g_param_spec_object ("individual-manager", "individual-manager",
+      "EmpathyIndividualManager",
+      EMPATHY_TYPE_INDIVIDUAL_MANAGER,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INDIVIDUAL_MGR, spec);
+
   g_type_class_add_private (object_class, sizeof (EmpathyChatWindowPriv));
 }
 
@@ -2378,6 +2466,8 @@ empathy_chat_window_init (EmpathyChatWindow *self)
       G_CALLBACK (chat_window_delete_event_cb), self);
   g_signal_connect (self->priv->dialog, "focus_in_event",
       G_CALLBACK (chat_window_focus_in_event_cb), self);
+  g_signal_connect (self->priv->dialog, "focus_out_event",
+      G_CALLBACK (chat_window_focus_out_event_cb), self);
   g_signal_connect_after (self->priv->notebook, "switch_page",
       G_CALLBACK (chat_window_page_switched_cb), self);
   g_signal_connect (self->priv->notebook, "page_added",
@@ -2657,14 +2747,14 @@ empathy_chat_window_find_chat (TpAccount *account,
   return NULL;
 }
 
-void
+EmpathyChatWindow *
 empathy_chat_window_present_chat (EmpathyChat *chat,
     gint64 timestamp)
 {
   EmpathyChatWindow *self;
   guint32 x_timestamp;
 
-  g_return_if_fail (EMPATHY_IS_CHAT (chat));
+  g_return_val_if_fail (EMPATHY_IS_CHAT (chat), NULL);
 
   self = chat_window_find_chat (chat);
 
@@ -2688,7 +2778,7 @@ empathy_chat_window_present_chat (EmpathyChat *chat,
    * an action by the user
    */
   if (!tp_user_action_time_should_present (timestamp, &x_timestamp))
-    return;
+    return self;
 
   if (x_timestamp != GDK_CURRENT_TIME)
     {
@@ -2698,7 +2788,7 @@ empathy_chat_window_present_chat (EmpathyChat *chat,
 
       if (self->priv->x_user_action_time != 0
         && X_EARLIER_OR_EQL (x_timestamp, self->priv->x_user_action_time))
-        return;
+        return self;
 
       self->priv->x_user_action_time = x_timestamp;
     }
@@ -2712,6 +2802,7 @@ empathy_chat_window_present_chat (EmpathyChat *chat,
   empathy_move_to_window_desktop (GTK_WINDOW (self->priv->dialog), x_timestamp);
 
   gtk_widget_grab_focus (chat->input_text_view);
+  return self;
 }
 
 static void
@@ -2735,3 +2826,9 @@ empathy_chat_window_get_nb_chats (EmpathyChatWindow *self,
   if (nb_private != NULL)
     *nb_private = _nb_private;
 }
+
+EmpathyIndividualManager *
+empathy_chat_window_get_individual_manager (EmpathyChatWindow *self)
+{
+  return self->priv->individual_mgr;
+}
